/*=========================================================================

  Program:   Visualization Toolkit
  Module:    TestLightWidget.cxx

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
//
// This example tests the vtkLightWidget

#include <vtkActor.h>
#include <vtkCamera.h>
#include <vtkInteractorEventRecorder.h>
#include <vtkInteractorStyleTrackballCamera.h>
#include <vtkLightRepresentation.h>
#include <vtkLightWidget.h>
#include <vtkNew.h>
#include <vtkRenderWindow.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkRenderer.h>

const char eventLog[] = "# StreamVersion 1.1\n"
                        "KeyReleaseEvent 675 -515 0 13 1 KP_Enter\n"
                        "EnterEvent 242 0 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 242 0 0 0 0 KP_Enter\n"
                        "RenderEvent 242 0 0 0 0 KP_Enter\n"
                        "RenderEvent 242 0 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 222 16 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 216 20 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 208 26 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 202 30 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 196 34 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 190 38 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 182 44 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 48 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 52 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 164 56 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 158 60 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 66 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 144 70 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 138 74 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 132 78 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 126 82 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 120 86 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 114 90 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 108 94 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 104 98 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 96 102 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 92 106 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 84 110 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 80 114 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 74 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 68 122 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 64 124 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 58 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 52 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 46 134 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 42 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 36 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 32 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 26 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 20 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 16 152 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 10 154 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 6 158 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 0 160 0 0 0 KP_Enter\n"
                        "LeaveEvent -2 162 0 0 0 KP_Enter\n"
                        "EnterEvent 0 224 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 0 224 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 1 224 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 1 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 2 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 4 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 5 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 5 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 7 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 8 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 9 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 10 228 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 12 228 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 13 228 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 15 228 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 16 228 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 18 230 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 19 230 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 20 230 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 22 230 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 23 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 25 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 28 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 30 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 33 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 34 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 36 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 39 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 41 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 45 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 49 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 50 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 52 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 55 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 59 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 61 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 63 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 67 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 69 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 71 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 73 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 74 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 76 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 80 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 82 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 84 229 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 88 229 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 90 229 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 91 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 95 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 97 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 99 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 103 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 105 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 107 224 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 109 224 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 111 222 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 115 222 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 220 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 119 220 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 120 220 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 122 218 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 124 218 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 126 216 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 128 216 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 130 214 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 132 212 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 134 212 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 135 210 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 137 210 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 137 209 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 139 209 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 141 209 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 141 207 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 142 205 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 144 205 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 144 203 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 146 203 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 201 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 200 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 151 198 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 153 196 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 196 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 195 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 156 193 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 193 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 192 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 190 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 159 190 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 159 188 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 161 188 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 161 187 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 162 185 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 164 184 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 164 182 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 181 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 180 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 179 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 177 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 176 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 175 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 174 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 173 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 172 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 171 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 170 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 169 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 168 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 164 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 164 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 163 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 162 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 160 164 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 159 164 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 162 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 156 162 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 155 162 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 155 160 0 0 0 KP_Enter\n"
                        "RenderEvent 155 160 0 0 0 KP_Enter\n"
                        "RenderEvent 155 160 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 157 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 152 157 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 152 155 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 152 154 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 152 153 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 152 151 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 152 150 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 150 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 149 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 155 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 155 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 158 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 158 139 0 0 0 KP_Enter\n"
                        "RenderEvent 158 139 0 0 0 KP_Enter\n"
                        "RenderEvent 158 139 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 127 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 125 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 123 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 122 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 120 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 120 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 179 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 181 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 182 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 114 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 185 114 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 114 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 112 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 188 112 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 189 112 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 190 112 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 190 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 190 114 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 189 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 189 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 188 117 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 188 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 120 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 121 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 185 121 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 185 122 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 123 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 125 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 183 126 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 183 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 181 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 181 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 179 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 179 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 135 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 135 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 136 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 139 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 139 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 142 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 142 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 164 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 163 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 163 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 162 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 161 146 0 0 0 KP_Enter\n"
                        "RenderEvent 161 146 0 0 0 KP_Enter\n"
                        "RenderEvent 161 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 159 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 158 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 158 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 150 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 151 0 0 0 KP_Enter\n"
                        "LeftButtonPressEvent 157 151 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 156 151 0 0 0 KP_Enter\n"
                        "RenderEvent 156 151 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 156 153 0 0 0 KP_Enter\n"
                        "RenderEvent 156 153 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 158 0 0 0 KP_Enter\n"
                        "RenderEvent 154 158 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 153 162 0 0 0 KP_Enter\n"
                        "RenderEvent 153 162 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 153 168 0 0 0 KP_Enter\n"
                        "RenderEvent 153 168 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 173 0 0 0 KP_Enter\n"
                        "RenderEvent 154 173 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 178 0 0 0 KP_Enter\n"
                        "RenderEvent 157 178 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 161 186 0 0 0 KP_Enter\n"
                        "RenderEvent 161 186 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 190 0 0 0 KP_Enter\n"
                        "RenderEvent 165 190 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 197 0 0 0 KP_Enter\n"
                        "RenderEvent 172 197 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 202 0 0 0 KP_Enter\n"
                        "RenderEvent 177 202 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 182 207 0 0 0 KP_Enter\n"
                        "RenderEvent 182 207 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 185 210 0 0 0 KP_Enter\n"
                        "RenderEvent 185 210 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 191 215 0 0 0 KP_Enter\n"
                        "RenderEvent 191 215 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 196 219 0 0 0 KP_Enter\n"
                        "RenderEvent 196 219 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 203 222 0 0 0 KP_Enter\n"
                        "RenderEvent 203 222 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 209 227 0 0 0 KP_Enter\n"
                        "RenderEvent 209 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 218 232 0 0 0 KP_Enter\n"
                        "RenderEvent 218 232 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 223 234 0 0 0 KP_Enter\n"
                        "RenderEvent 223 234 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 228 238 0 0 0 KP_Enter\n"
                        "RenderEvent 228 238 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 231 239 0 0 0 KP_Enter\n"
                        "RenderEvent 231 239 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 232 240 0 0 0 KP_Enter\n"
                        "RenderEvent 232 240 0 0 0 KP_Enter\n"
                        "LeftButtonReleaseEvent 232 240 0 0 0 KP_Enter\n"
                        "RenderEvent 232 240 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 231 240 0 0 0 KP_Enter\n"
                        "RenderEvent 231 240 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 230 240 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 230 239 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 229 239 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 229 238 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 227 238 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 226 238 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 226 236 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 224 236 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 223 234 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 223 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 221 233 0 0 0 KP_Enter\n"
                        "RenderEvent 221 233 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 219 231 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 218 229 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 216 229 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 216 228 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 214 226 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 213 226 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 213 224 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 211 224 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 211 222 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 209 222 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 209 221 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 207 219 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 205 219 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 205 217 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 204 215 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 202 214 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 200 212 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 200 210 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 199 209 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 197 207 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 197 205 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 195 205 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 195 204 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 195 202 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 193 200 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 192 199 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 192 197 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 190 195 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 190 194 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 188 192 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 188 190 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 189 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 187 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 185 186 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 185 184 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 184 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 182 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 181 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 182 179 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 182 178 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 178 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 176 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 175 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 179 173 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 179 172 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 172 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 170 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 169 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 169 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 175 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 175 164 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 175 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 161 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 160 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 159 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 158 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 158 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 157 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 156 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 155 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 154 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 153 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 152 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 151 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 151 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 150 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 149 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 149 0 0 0 KP_Enter\n"
                        "LeftButtonPressEvent 171 149 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 149 0 0 0 KP_Enter\n"
                        "RenderEvent 172 149 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 149 0 0 0 KP_Enter\n"
                        "RenderEvent 172 149 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 148 0 0 0 KP_Enter\n"
                        "RenderEvent 172 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 148 0 0 0 KP_Enter\n"
                        "RenderEvent 173 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 145 0 0 0 KP_Enter\n"
                        "RenderEvent 176 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 179 144 0 0 0 KP_Enter\n"
                        "RenderEvent 179 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 181 142 0 0 0 KP_Enter\n"
                        "RenderEvent 181 142 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 186 140 0 0 0 KP_Enter\n"
                        "RenderEvent 186 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 191 138 0 0 0 KP_Enter\n"
                        "RenderEvent 191 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 195 136 0 0 0 KP_Enter\n"
                        "RenderEvent 195 136 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 202 134 0 0 0 KP_Enter\n"
                        "RenderEvent 202 134 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 206 133 0 0 0 KP_Enter\n"
                        "RenderEvent 206 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 210 132 0 0 0 KP_Enter\n"
                        "RenderEvent 210 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 213 131 0 0 0 KP_Enter\n"
                        "RenderEvent 213 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 215 131 0 0 0 KP_Enter\n"
                        "RenderEvent 215 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 219 130 0 0 0 KP_Enter\n"
                        "RenderEvent 219 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 221 130 0 0 0 KP_Enter\n"
                        "RenderEvent 221 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 223 129 0 0 0 KP_Enter\n"
                        "RenderEvent 223 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 225 129 0 0 0 KP_Enter\n"
                        "RenderEvent 225 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 227 129 0 0 0 KP_Enter\n"
                        "RenderEvent 227 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 229 130 0 0 0 KP_Enter\n"
                        "RenderEvent 229 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 231 130 0 0 0 KP_Enter\n"
                        "RenderEvent 231 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 234 130 0 0 0 KP_Enter\n"
                        "RenderEvent 234 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 235 130 0 0 0 KP_Enter\n"
                        "RenderEvent 235 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 236 131 0 0 0 KP_Enter\n"
                        "RenderEvent 236 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 237 131 0 0 0 KP_Enter\n"
                        "RenderEvent 237 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 238 131 0 0 0 KP_Enter\n"
                        "RenderEvent 238 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 238 131 0 0 0 KP_Enter\n"
                        "RenderEvent 238 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 239 131 0 0 0 KP_Enter\n"
                        "RenderEvent 239 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 240 131 0 0 0 KP_Enter\n"
                        "RenderEvent 240 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 242 132 0 0 0 KP_Enter\n"
                        "RenderEvent 242 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 243 132 0 0 0 KP_Enter\n"
                        "RenderEvent 243 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 243 132 0 0 0 KP_Enter\n"
                        "RenderEvent 243 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 244 131 0 0 0 KP_Enter\n"
                        "RenderEvent 244 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 244 130 0 0 0 KP_Enter\n"
                        "RenderEvent 244 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 244 130 0 0 0 KP_Enter\n"
                        "RenderEvent 244 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 244 129 0 0 0 KP_Enter\n"
                        "RenderEvent 244 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 244 128 0 0 0 KP_Enter\n"
                        "RenderEvent 244 128 0 0 0 KP_Enter\n"
                        "LeftButtonReleaseEvent 244 128 0 0 0 KP_Enter\n"
                        "RenderEvent 244 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 244 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 244 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 243 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 242 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 241 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 240 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 239 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 238 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 237 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 235 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 234 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 232 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 230 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 228 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 227 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 226 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 223 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 221 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 220 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 218 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 216 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 215 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 213 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 210 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 206 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 205 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 203 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 201 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 200 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 198 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 196 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 195 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 193 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 189 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 186 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 182 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 181 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 179 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 175 129 0 0 0 KP_Enter\n"
                        "RenderEvent 175 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 164 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 163 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 162 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 160 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 158 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 156 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 152 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 151 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 149 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 134 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 135 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 135 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 149 135 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 135 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 136 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 137 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 151 137 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 151 138 0 0 0 KP_Enter\n"
                        "RenderEvent 151 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 151 139 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 151 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 151 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 151 142 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 142 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 149 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 149 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 149 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 147 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 147 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 146 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 146 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 145 148 0 0 0 KP_Enter\n"
                        "LeftButtonPressEvent 145 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 144 148 0 0 0 KP_Enter\n"
                        "RenderEvent 144 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 144 149 0 0 0 KP_Enter\n"
                        "RenderEvent 144 149 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 142 150 0 0 0 KP_Enter\n"
                        "RenderEvent 142 150 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 138 152 0 0 0 KP_Enter\n"
                        "RenderEvent 138 152 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 133 156 0 0 0 KP_Enter\n"
                        "RenderEvent 133 156 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 127 163 0 0 0 KP_Enter\n"
                        "RenderEvent 127 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 121 167 0 0 0 KP_Enter\n"
                        "RenderEvent 121 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 172 0 0 0 KP_Enter\n"
                        "RenderEvent 116 172 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 111 179 0 0 0 KP_Enter\n"
                        "RenderEvent 111 179 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 104 187 0 0 0 KP_Enter\n"
                        "RenderEvent 104 187 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 98 195 0 0 0 KP_Enter\n"
                        "RenderEvent 98 195 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 93 201 0 0 0 KP_Enter\n"
                        "RenderEvent 93 201 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 91 206 0 0 0 KP_Enter\n"
                        "RenderEvent 91 206 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 89 208 0 0 0 KP_Enter\n"
                        "RenderEvent 89 208 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 88 209 0 0 0 KP_Enter\n"
                        "RenderEvent 88 209 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 87 212 0 0 0 KP_Enter\n"
                        "RenderEvent 87 212 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 84 214 0 0 0 KP_Enter\n"
                        "RenderEvent 84 214 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 83 218 0 0 0 KP_Enter\n"
                        "RenderEvent 83 218 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 81 221 0 0 0 KP_Enter\n"
                        "RenderEvent 81 221 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 79 223 0 0 0 KP_Enter\n"
                        "RenderEvent 79 223 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 77 226 0 0 0 KP_Enter\n"
                        "RenderEvent 77 226 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 76 227 0 0 0 KP_Enter\n"
                        "RenderEvent 76 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 75 229 0 0 0 KP_Enter\n"
                        "RenderEvent 75 229 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 75 230 0 0 0 KP_Enter\n"
                        "RenderEvent 75 230 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 74 230 0 0 0 KP_Enter\n"
                        "RenderEvent 74 230 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 74 230 0 0 0 KP_Enter\n"
                        "RenderEvent 74 230 0 0 0 KP_Enter\n"
                        "LeftButtonReleaseEvent 74 230 0 0 0 KP_Enter\n"
                        "RenderEvent 74 230 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 74 230 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 75 229 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 75 228 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 76 226 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 77 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 77 224 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 79 222 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 80 221 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 82 218 0 0 0 KP_Enter\n"
                        "RenderEvent 82 218 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 85 213 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 86 212 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 88 209 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 89 207 0 0 0 KP_Enter\n"
                        "RenderEvent 89 207 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 91 204 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 91 203 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 92 203 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 92 201 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 94 201 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 94 199 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 94 198 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 95 198 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 95 196 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 97 195 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 97 193 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 97 192 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 98 192 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 98 190 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 98 189 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 99 187 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 99 186 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 101 186 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 101 185 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 101 183 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 101 182 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 102 182 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 102 181 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 103 181 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 103 180 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 103 179 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 104 179 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 105 179 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 105 178 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 106 177 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 107 177 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 107 176 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 108 176 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 109 175 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 110 175 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 110 174 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 111 174 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 111 173 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 112 173 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 114 173 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 114 172 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 115 172 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 115 171 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 171 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 171 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 170 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 118 170 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 118 168 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 119 168 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 120 168 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 121 168 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 122 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 123 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 124 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 125 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 126 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 127 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 128 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 129 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 130 165 0 0 0 KP_Enter\n"
                        "RenderEvent 130 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 131 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 132 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 133 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 133 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 133 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 134 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 134 164 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 135 164 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 136 164 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 136 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 137 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 138 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 139 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 139 162 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 140 162 0 0 0 KP_Enter\n"
                        "LeftButtonPressEvent 140 162 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 140 161 0 0 0 KP_Enter\n"
                        "RenderEvent 140 161 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 140 160 0 0 0 KP_Enter\n"
                        "RenderEvent 140 160 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 139 160 0 0 0 KP_Enter\n"
                        "RenderEvent 139 160 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 137 157 0 0 0 KP_Enter\n"
                        "RenderEvent 137 157 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 134 155 0 0 0 KP_Enter\n"
                        "RenderEvent 134 155 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 129 152 0 0 0 KP_Enter\n"
                        "RenderEvent 129 152 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 126 149 0 0 0 KP_Enter\n"
                        "RenderEvent 126 149 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 122 147 0 0 0 KP_Enter\n"
                        "RenderEvent 122 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 146 0 0 0 KP_Enter\n"
                        "RenderEvent 117 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 112 144 0 0 0 KP_Enter\n"
                        "RenderEvent 112 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 109 142 0 0 0 KP_Enter\n"
                        "RenderEvent 109 142 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 104 140 0 0 0 KP_Enter\n"
                        "RenderEvent 104 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 99 139 0 0 0 KP_Enter\n"
                        "RenderEvent 99 139 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 94 137 0 0 0 KP_Enter\n"
                        "RenderEvent 94 137 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 90 135 0 0 0 KP_Enter\n"
                        "RenderEvent 90 135 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 83 134 0 0 0 KP_Enter\n"
                        "RenderEvent 83 134 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 80 132 0 0 0 KP_Enter\n"
                        "RenderEvent 80 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 77 130 0 0 0 KP_Enter\n"
                        "RenderEvent 77 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 73 129 0 0 0 KP_Enter\n"
                        "RenderEvent 73 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 68 127 0 0 0 KP_Enter\n"
                        "RenderEvent 68 127 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 64 125 0 0 0 KP_Enter\n"
                        "RenderEvent 64 125 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 59 121 0 0 0 KP_Enter\n"
                        "RenderEvent 59 121 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 55 119 0 0 0 KP_Enter\n"
                        "RenderEvent 55 119 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 50 117 0 0 0 KP_Enter\n"
                        "RenderEvent 50 117 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 48 116 0 0 0 KP_Enter\n"
                        "RenderEvent 48 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 47 115 0 0 0 KP_Enter\n"
                        "RenderEvent 47 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 46 115 0 0 0 KP_Enter\n"
                        "RenderEvent 46 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 46 115 0 0 0 KP_Enter\n"
                        "RenderEvent 46 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 47 115 0 0 0 KP_Enter\n"
                        "RenderEvent 47 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 53 114 0 0 0 KP_Enter\n"
                        "RenderEvent 53 114 0 0 0 KP_Enter\n"
                        "LeftButtonReleaseEvent 48 115 0 0 0 KP_Enter\n"
                        "RenderEvent 48 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 91 111 0 0 0 KP_Enter\n"
                        "RenderEvent 91 111 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 118 111 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 122 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 123 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 123 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 125 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 127 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 128 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 130 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 132 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 133 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 135 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 137 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 139 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 140 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 141 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 143 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 145 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 146 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 151 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 153 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 155 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 156 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 158 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 159 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 159 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 161 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 162 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 163 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 119 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 119 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 119 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 119 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 120 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 120 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 121 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 121 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 122 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 122 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 122 0 0 0 KP_Enter\n"
                        "RenderEvent 173 122 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 124 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 175 124 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 175 125 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 125 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 178 125 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 178 126 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 179 126 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 179 127 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 127 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 181 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 181 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 182 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 183 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 185 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 186 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 186 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 134 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 188 134 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 188 135 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 189 135 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 189 136 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 190 136 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 191 136 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 191 137 0 0 0 KP_Enter\n"
                        "RightButtonPressEvent 191 137 0 0 0 KP_Enter\n"
                        "RenderEvent 191 137 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 191 138 0 0 0 KP_Enter\n"
                        "RenderEvent 191 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 191 138 0 0 0 KP_Enter\n"
                        "RenderEvent 191 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 190 138 0 0 0 KP_Enter\n"
                        "RenderEvent 190 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 189 138 0 0 0 KP_Enter\n"
                        "RenderEvent 189 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 189 139 0 0 0 KP_Enter\n"
                        "RenderEvent 189 139 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 188 139 0 0 0 KP_Enter\n"
                        "RenderEvent 188 139 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 187 139 0 0 0 KP_Enter\n"
                        "RenderEvent 187 139 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 186 140 0 0 0 KP_Enter\n"
                        "RenderEvent 186 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 185 140 0 0 0 KP_Enter\n"
                        "RenderEvent 185 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 185 140 0 0 0 KP_Enter\n"
                        "RenderEvent 185 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 184 140 0 0 0 KP_Enter\n"
                        "RenderEvent 184 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 183 140 0 0 0 KP_Enter\n"
                        "RenderEvent 183 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 182 140 0 0 0 KP_Enter\n"
                        "RenderEvent 182 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 181 141 0 0 0 KP_Enter\n"
                        "RenderEvent 181 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 141 0 0 0 KP_Enter\n"
                        "RenderEvent 180 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 180 141 0 0 0 KP_Enter\n"
                        "RenderEvent 180 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 179 141 0 0 0 KP_Enter\n"
                        "RenderEvent 179 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 178 141 0 0 0 KP_Enter\n"
                        "RenderEvent 178 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 141 0 0 0 KP_Enter\n"
                        "RenderEvent 177 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 142 0 0 0 KP_Enter\n"
                        "RenderEvent 177 142 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 142 0 0 0 KP_Enter\n"
                        "RenderEvent 174 142 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 143 0 0 0 KP_Enter\n"
                        "RenderEvent 171 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 143 0 0 0 KP_Enter\n"
                        "RenderEvent 170 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 143 0 0 0 KP_Enter\n"
                        "RenderEvent 169 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 143 0 0 0 KP_Enter\n"
                        "RenderEvent 169 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 143 0 0 0 KP_Enter\n"
                        "RenderEvent 168 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 143 0 0 0 KP_Enter\n"
                        "RenderEvent 167 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 164 144 0 0 0 KP_Enter\n"
                        "RenderEvent 164 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 161 144 0 0 0 KP_Enter\n"
                        "RenderEvent 161 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 159 145 0 0 0 KP_Enter\n"
                        "RenderEvent 159 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 145 0 0 0 KP_Enter\n"
                        "RenderEvent 157 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 156 145 0 0 0 KP_Enter\n"
                        "RenderEvent 156 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 155 145 0 0 0 KP_Enter\n"
                        "RenderEvent 155 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 145 0 0 0 KP_Enter\n"
                        "RenderEvent 154 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 152 146 0 0 0 KP_Enter\n"
                        "RenderEvent 152 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 146 0 0 0 KP_Enter\n"
                        "RenderEvent 150 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 146 0 0 0 KP_Enter\n"
                        "RenderEvent 150 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 149 146 0 0 0 KP_Enter\n"
                        "RenderEvent 149 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 149 147 0 0 0 KP_Enter\n"
                        "RenderEvent 149 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 147 0 0 0 KP_Enter\n"
                        "RenderEvent 148 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 147 147 0 0 0 KP_Enter\n"
                        "RenderEvent 147 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 145 147 0 0 0 KP_Enter\n"
                        "RenderEvent 145 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 143 147 0 0 0 KP_Enter\n"
                        "RenderEvent 143 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 142 148 0 0 0 KP_Enter\n"
                        "RenderEvent 142 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 141 148 0 0 0 KP_Enter\n"
                        "RenderEvent 141 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 140 148 0 0 0 KP_Enter\n"
                        "RenderEvent 140 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 139 148 0 0 0 KP_Enter\n"
                        "RenderEvent 139 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 137 149 0 0 0 KP_Enter\n"
                        "RenderEvent 137 149 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 136 150 0 0 0 KP_Enter\n"
                        "RenderEvent 136 150 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 135 150 0 0 0 KP_Enter\n"
                        "RenderEvent 135 150 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 135 150 0 0 0 KP_Enter\n"
                        "RenderEvent 135 150 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 134 151 0 0 0 KP_Enter\n"
                        "RenderEvent 134 151 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 133 151 0 0 0 KP_Enter\n"
                        "RenderEvent 133 151 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 132 152 0 0 0 KP_Enter\n"
                        "RenderEvent 132 152 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 132 153 0 0 0 KP_Enter\n"
                        "RenderEvent 132 153 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 131 153 0 0 0 KP_Enter\n"
                        "RenderEvent 131 153 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 131 154 0 0 0 KP_Enter\n"
                        "RenderEvent 131 154 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 130 155 0 0 0 KP_Enter\n"
                        "RenderEvent 130 155 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 129 156 0 0 0 KP_Enter\n"
                        "RenderEvent 129 156 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 129 157 0 0 0 KP_Enter\n"
                        "RenderEvent 129 157 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 129 158 0 0 0 KP_Enter\n"
                        "RenderEvent 129 158 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 128 158 0 0 0 KP_Enter\n"
                        "RenderEvent 128 158 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 127 159 0 0 0 KP_Enter\n"
                        "RenderEvent 127 159 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 127 160 0 0 0 KP_Enter\n"
                        "RenderEvent 127 160 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 126 161 0 0 0 KP_Enter\n"
                        "RenderEvent 126 161 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 126 162 0 0 0 KP_Enter\n"
                        "RenderEvent 126 162 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 126 163 0 0 0 KP_Enter\n"
                        "RenderEvent 126 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 125 163 0 0 0 KP_Enter\n"
                        "RenderEvent 125 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 125 165 0 0 0 KP_Enter\n"
                        "RenderEvent 125 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 124 166 0 0 0 KP_Enter\n"
                        "RenderEvent 124 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 124 167 0 0 0 KP_Enter\n"
                        "RenderEvent 124 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 123 167 0 0 0 KP_Enter\n"
                        "RenderEvent 123 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 123 168 0 0 0 KP_Enter\n"
                        "RenderEvent 123 168 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 123 169 0 0 0 KP_Enter\n"
                        "RenderEvent 123 169 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 122 169 0 0 0 KP_Enter\n"
                        "RenderEvent 122 169 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 122 170 0 0 0 KP_Enter\n"
                        "RenderEvent 122 170 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 122 171 0 0 0 KP_Enter\n"
                        "RenderEvent 122 171 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 121 172 0 0 0 KP_Enter\n"
                        "RenderEvent 121 172 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 121 173 0 0 0 KP_Enter\n"
                        "RenderEvent 121 173 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 120 175 0 0 0 KP_Enter\n"
                        "RenderEvent 120 175 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 119 176 0 0 0 KP_Enter\n"
                        "RenderEvent 119 176 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 119 177 0 0 0 KP_Enter\n"
                        "RenderEvent 119 177 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 119 178 0 0 0 KP_Enter\n"
                        "RenderEvent 119 178 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 118 180 0 0 0 KP_Enter\n"
                        "RenderEvent 118 180 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 118 181 0 0 0 KP_Enter\n"
                        "RenderEvent 118 181 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 182 0 0 0 KP_Enter\n"
                        "RenderEvent 117 182 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 183 0 0 0 KP_Enter\n"
                        "RenderEvent 117 183 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 185 0 0 0 KP_Enter\n"
                        "RenderEvent 117 185 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 186 0 0 0 KP_Enter\n"
                        "RenderEvent 116 186 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 188 0 0 0 KP_Enter\n"
                        "RenderEvent 116 188 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 189 0 0 0 KP_Enter\n"
                        "RenderEvent 116 189 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 190 0 0 0 KP_Enter\n"
                        "RenderEvent 116 190 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 192 0 0 0 KP_Enter\n"
                        "RenderEvent 116 192 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 193 0 0 0 KP_Enter\n"
                        "RenderEvent 116 193 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 194 0 0 0 KP_Enter\n"
                        "RenderEvent 116 194 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 195 0 0 0 KP_Enter\n"
                        "RenderEvent 116 195 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 197 0 0 0 KP_Enter\n"
                        "RenderEvent 116 197 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 198 0 0 0 KP_Enter\n"
                        "RenderEvent 116 198 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 199 0 0 0 KP_Enter\n"
                        "RenderEvent 116 199 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 200 0 0 0 KP_Enter\n"
                        "RenderEvent 116 200 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 201 0 0 0 KP_Enter\n"
                        "RenderEvent 116 201 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 202 0 0 0 KP_Enter\n"
                        "RenderEvent 116 202 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 203 0 0 0 KP_Enter\n"
                        "RenderEvent 116 203 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 205 0 0 0 KP_Enter\n"
                        "RenderEvent 116 205 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 206 0 0 0 KP_Enter\n"
                        "RenderEvent 116 206 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 207 0 0 0 KP_Enter\n"
                        "RenderEvent 116 207 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 209 0 0 0 KP_Enter\n"
                        "RenderEvent 116 209 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 211 0 0 0 KP_Enter\n"
                        "RenderEvent 116 211 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 212 0 0 0 KP_Enter\n"
                        "RenderEvent 116 212 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 213 0 0 0 KP_Enter\n"
                        "RenderEvent 116 213 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 214 0 0 0 KP_Enter\n"
                        "RenderEvent 116 214 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 214 0 0 0 KP_Enter\n"
                        "RenderEvent 116 214 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 215 0 0 0 KP_Enter\n"
                        "RenderEvent 116 215 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 116 216 0 0 0 KP_Enter\n"
                        "RenderEvent 116 216 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 216 0 0 0 KP_Enter\n"
                        "RenderEvent 117 216 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 217 0 0 0 KP_Enter\n"
                        "RenderEvent 117 217 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 218 0 0 0 KP_Enter\n"
                        "RenderEvent 117 218 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 219 0 0 0 KP_Enter\n"
                        "RenderEvent 117 219 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 220 0 0 0 KP_Enter\n"
                        "RenderEvent 117 220 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 220 0 0 0 KP_Enter\n"
                        "RenderEvent 117 220 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 221 0 0 0 KP_Enter\n"
                        "RenderEvent 117 221 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 222 0 0 0 KP_Enter\n"
                        "RenderEvent 117 222 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 223 0 0 0 KP_Enter\n"
                        "RenderEvent 117 223 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 223 0 0 0 KP_Enter\n"
                        "RenderEvent 117 223 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 224 0 0 0 KP_Enter\n"
                        "RenderEvent 117 224 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 225 0 0 0 KP_Enter\n"
                        "RenderEvent 117 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 226 0 0 0 KP_Enter\n"
                        "RenderEvent 117 226 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 226 0 0 0 KP_Enter\n"
                        "RenderEvent 117 226 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 227 0 0 0 KP_Enter\n"
                        "RenderEvent 117 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 227 0 0 0 KP_Enter\n"
                        "RenderEvent 117 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 228 0 0 0 KP_Enter\n"
                        "RenderEvent 117 228 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 229 0 0 0 KP_Enter\n"
                        "RenderEvent 117 229 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 229 0 0 0 KP_Enter\n"
                        "RenderEvent 117 229 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 229 0 0 0 KP_Enter\n"
                        "RenderEvent 117 229 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 228 0 0 0 KP_Enter\n"
                        "RenderEvent 117 228 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 228 0 0 0 KP_Enter\n"
                        "RenderEvent 117 228 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 227 0 0 0 KP_Enter\n"
                        "RenderEvent 117 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 227 0 0 0 KP_Enter\n"
                        "RenderEvent 117 227 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 226 0 0 0 KP_Enter\n"
                        "RenderEvent 117 226 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 226 0 0 0 KP_Enter\n"
                        "RenderEvent 117 226 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 225 0 0 0 KP_Enter\n"
                        "RenderEvent 117 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 225 0 0 0 KP_Enter\n"
                        "RenderEvent 117 225 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 224 0 0 0 KP_Enter\n"
                        "RenderEvent 117 224 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 223 0 0 0 KP_Enter\n"
                        "RenderEvent 117 223 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 222 0 0 0 KP_Enter\n"
                        "RenderEvent 117 222 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 221 0 0 0 KP_Enter\n"
                        "RenderEvent 117 221 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 220 0 0 0 KP_Enter\n"
                        "RenderEvent 117 220 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 219 0 0 0 KP_Enter\n"
                        "RenderEvent 117 219 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 219 0 0 0 KP_Enter\n"
                        "RenderEvent 117 219 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 218 0 0 0 KP_Enter\n"
                        "RenderEvent 117 218 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 217 0 0 0 KP_Enter\n"
                        "RenderEvent 117 217 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 216 0 0 0 KP_Enter\n"
                        "RenderEvent 117 216 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 216 0 0 0 KP_Enter\n"
                        "RenderEvent 117 216 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 117 215 0 0 0 KP_Enter\n"
                        "RenderEvent 117 215 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 118 214 0 0 0 KP_Enter\n"
                        "RenderEvent 118 214 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 118 213 0 0 0 KP_Enter\n"
                        "RenderEvent 118 213 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 118 212 0 0 0 KP_Enter\n"
                        "RenderEvent 118 212 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 118 211 0 0 0 KP_Enter\n"
                        "RenderEvent 118 211 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 119 211 0 0 0 KP_Enter\n"
                        "RenderEvent 119 211 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 119 210 0 0 0 KP_Enter\n"
                        "RenderEvent 119 210 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 119 209 0 0 0 KP_Enter\n"
                        "RenderEvent 119 209 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 119 208 0 0 0 KP_Enter\n"
                        "RenderEvent 119 208 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 120 207 0 0 0 KP_Enter\n"
                        "RenderEvent 120 207 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 120 206 0 0 0 KP_Enter\n"
                        "RenderEvent 120 206 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 121 206 0 0 0 KP_Enter\n"
                        "RenderEvent 121 206 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 121 204 0 0 0 KP_Enter\n"
                        "RenderEvent 121 204 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 122 203 0 0 0 KP_Enter\n"
                        "RenderEvent 122 203 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 123 202 0 0 0 KP_Enter\n"
                        "RenderEvent 123 202 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 123 201 0 0 0 KP_Enter\n"
                        "RenderEvent 123 201 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 123 201 0 0 0 KP_Enter\n"
                        "RenderEvent 123 201 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 124 201 0 0 0 KP_Enter\n"
                        "RenderEvent 124 201 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 124 200 0 0 0 KP_Enter\n"
                        "RenderEvent 124 200 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 124 199 0 0 0 KP_Enter\n"
                        "RenderEvent 124 199 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 125 199 0 0 0 KP_Enter\n"
                        "RenderEvent 125 199 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 125 198 0 0 0 KP_Enter\n"
                        "RenderEvent 125 198 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 125 197 0 0 0 KP_Enter\n"
                        "RenderEvent 125 197 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 125 196 0 0 0 KP_Enter\n"
                        "RenderEvent 125 196 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 126 196 0 0 0 KP_Enter\n"
                        "RenderEvent 126 196 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 126 195 0 0 0 KP_Enter\n"
                        "RenderEvent 126 195 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 127 194 0 0 0 KP_Enter\n"
                        "RenderEvent 127 194 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 127 192 0 0 0 KP_Enter\n"
                        "RenderEvent 127 192 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 127 191 0 0 0 KP_Enter\n"
                        "RenderEvent 127 191 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 128 191 0 0 0 KP_Enter\n"
                        "RenderEvent 128 191 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 128 190 0 0 0 KP_Enter\n"
                        "RenderEvent 128 190 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 128 189 0 0 0 KP_Enter\n"
                        "RenderEvent 128 189 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 129 189 0 0 0 KP_Enter\n"
                        "RenderEvent 129 189 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 129 188 0 0 0 KP_Enter\n"
                        "RenderEvent 129 188 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 130 188 0 0 0 KP_Enter\n"
                        "RenderEvent 130 188 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 130 187 0 0 0 KP_Enter\n"
                        "RenderEvent 130 187 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 130 186 0 0 0 KP_Enter\n"
                        "RenderEvent 130 186 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 131 186 0 0 0 KP_Enter\n"
                        "RenderEvent 131 186 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 131 185 0 0 0 KP_Enter\n"
                        "RenderEvent 131 185 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 132 185 0 0 0 KP_Enter\n"
                        "RenderEvent 132 185 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 132 184 0 0 0 KP_Enter\n"
                        "RenderEvent 132 184 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 133 184 0 0 0 KP_Enter\n"
                        "RenderEvent 133 184 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 133 183 0 0 0 KP_Enter\n"
                        "RenderEvent 133 183 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 134 183 0 0 0 KP_Enter\n"
                        "RenderEvent 134 183 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 135 182 0 0 0 KP_Enter\n"
                        "RenderEvent 135 182 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 136 182 0 0 0 KP_Enter\n"
                        "RenderEvent 136 182 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 137 181 0 0 0 KP_Enter\n"
                        "RenderEvent 137 181 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 138 181 0 0 0 KP_Enter\n"
                        "RenderEvent 138 181 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 139 180 0 0 0 KP_Enter\n"
                        "RenderEvent 139 180 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 140 179 0 0 0 KP_Enter\n"
                        "RenderEvent 140 179 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 141 177 0 0 0 KP_Enter\n"
                        "RenderEvent 141 177 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 143 177 0 0 0 KP_Enter\n"
                        "RenderEvent 143 177 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 143 177 0 0 0 KP_Enter\n"
                        "RenderEvent 143 177 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 143 177 0 0 0 KP_Enter\n"
                        "RenderEvent 143 177 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 143 176 0 0 0 KP_Enter\n"
                        "RenderEvent 143 176 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 144 176 0 0 0 KP_Enter\n"
                        "RenderEvent 144 176 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 144 175 0 0 0 KP_Enter\n"
                        "RenderEvent 144 175 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 145 174 0 0 0 KP_Enter\n"
                        "RenderEvent 145 174 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 146 174 0 0 0 KP_Enter\n"
                        "RenderEvent 146 174 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 146 173 0 0 0 KP_Enter\n"
                        "RenderEvent 146 173 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 147 173 0 0 0 KP_Enter\n"
                        "RenderEvent 147 173 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 147 172 0 0 0 KP_Enter\n"
                        "RenderEvent 147 172 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 172 0 0 0 KP_Enter\n"
                        "RenderEvent 148 172 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 148 171 0 0 0 KP_Enter\n"
                        "RenderEvent 148 171 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 149 171 0 0 0 KP_Enter\n"
                        "RenderEvent 149 171 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 171 0 0 0 KP_Enter\n"
                        "RenderEvent 150 171 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 170 0 0 0 KP_Enter\n"
                        "RenderEvent 150 170 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 150 170 0 0 0 KP_Enter\n"
                        "RenderEvent 150 170 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 151 169 0 0 0 KP_Enter\n"
                        "RenderEvent 151 169 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 152 169 0 0 0 KP_Enter\n"
                        "RenderEvent 152 169 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 152 169 0 0 0 KP_Enter\n"
                        "RenderEvent 152 169 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 153 169 0 0 0 KP_Enter\n"
                        "RenderEvent 153 169 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 154 169 0 0 0 KP_Enter\n"
                        "RenderEvent 154 169 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 155 168 0 0 0 KP_Enter\n"
                        "RenderEvent 155 168 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 155 167 0 0 0 KP_Enter\n"
                        "RenderEvent 155 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 156 167 0 0 0 KP_Enter\n"
                        "RenderEvent 156 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 157 167 0 0 0 KP_Enter\n"
                        "RenderEvent 157 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 158 167 0 0 0 KP_Enter\n"
                        "RenderEvent 158 167 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 158 166 0 0 0 KP_Enter\n"
                        "RenderEvent 158 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 159 166 0 0 0 KP_Enter\n"
                        "RenderEvent 159 166 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 160 165 0 0 0 KP_Enter\n"
                        "RenderEvent 160 165 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 161 164 0 0 0 KP_Enter\n"
                        "RenderEvent 161 164 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 162 164 0 0 0 KP_Enter\n"
                        "RenderEvent 162 164 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 162 163 0 0 0 KP_Enter\n"
                        "RenderEvent 162 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 162 163 0 0 0 KP_Enter\n"
                        "RenderEvent 162 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 163 163 0 0 0 KP_Enter\n"
                        "RenderEvent 163 163 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 163 162 0 0 0 KP_Enter\n"
                        "RenderEvent 163 162 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 164 162 0 0 0 KP_Enter\n"
                        "RenderEvent 164 162 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 162 0 0 0 KP_Enter\n"
                        "RenderEvent 165 162 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 161 0 0 0 KP_Enter\n"
                        "RenderEvent 165 161 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 161 0 0 0 KP_Enter\n"
                        "RenderEvent 165 161 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 161 0 0 0 KP_Enter\n"
                        "RenderEvent 165 161 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 160 0 0 0 KP_Enter\n"
                        "RenderEvent 165 160 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 165 159 0 0 0 KP_Enter\n"
                        "RenderEvent 165 159 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 159 0 0 0 KP_Enter\n"
                        "RenderEvent 166 159 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 157 0 0 0 KP_Enter\n"
                        "RenderEvent 166 157 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 155 0 0 0 KP_Enter\n"
                        "RenderEvent 167 155 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 154 0 0 0 KP_Enter\n"
                        "RenderEvent 167 154 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 153 0 0 0 KP_Enter\n"
                        "RenderEvent 167 153 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 152 0 0 0 KP_Enter\n"
                        "RenderEvent 167 152 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 152 0 0 0 KP_Enter\n"
                        "RenderEvent 167 152 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 151 0 0 0 KP_Enter\n"
                        "RenderEvent 167 151 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 150 0 0 0 KP_Enter\n"
                        "RenderEvent 167 150 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 149 0 0 0 KP_Enter\n"
                        "RenderEvent 167 149 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 149 0 0 0 KP_Enter\n"
                        "RenderEvent 166 149 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 148 0 0 0 KP_Enter\n"
                        "RenderEvent 166 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 148 0 0 0 KP_Enter\n"
                        "RenderEvent 166 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 148 0 0 0 KP_Enter\n"
                        "RenderEvent 166 148 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 147 0 0 0 KP_Enter\n"
                        "RenderEvent 166 147 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 146 0 0 0 KP_Enter\n"
                        "RenderEvent 166 146 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 145 0 0 0 KP_Enter\n"
                        "RenderEvent 166 145 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 144 0 0 0 KP_Enter\n"
                        "RenderEvent 166 144 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 143 0 0 0 KP_Enter\n"
                        "RenderEvent 166 143 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 142 0 0 0 KP_Enter\n"
                        "RenderEvent 166 142 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 141 0 0 0 KP_Enter\n"
                        "RenderEvent 166 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 141 0 0 0 KP_Enter\n"
                        "RenderEvent 166 141 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 140 0 0 0 KP_Enter\n"
                        "RenderEvent 166 140 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 139 0 0 0 KP_Enter\n"
                        "RenderEvent 166 139 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 138 0 0 0 KP_Enter\n"
                        "RenderEvent 166 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 138 0 0 0 KP_Enter\n"
                        "RenderEvent 166 138 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 137 0 0 0 KP_Enter\n"
                        "RenderEvent 166 137 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 136 0 0 0 KP_Enter\n"
                        "RenderEvent 166 136 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 166 136 0 0 0 KP_Enter\n"
                        "RenderEvent 166 136 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 136 0 0 0 KP_Enter\n"
                        "RenderEvent 167 136 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 135 0 0 0 KP_Enter\n"
                        "RenderEvent 167 135 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 134 0 0 0 KP_Enter\n"
                        "RenderEvent 167 134 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 134 0 0 0 KP_Enter\n"
                        "RenderEvent 167 134 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 133 0 0 0 KP_Enter\n"
                        "RenderEvent 167 133 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 132 0 0 0 KP_Enter\n"
                        "RenderEvent 167 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 132 0 0 0 KP_Enter\n"
                        "RenderEvent 167 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 132 0 0 0 KP_Enter\n"
                        "RenderEvent 168 132 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 131 0 0 0 KP_Enter\n"
                        "RenderEvent 168 131 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 130 0 0 0 KP_Enter\n"
                        "RenderEvent 168 130 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 129 0 0 0 KP_Enter\n"
                        "RenderEvent 168 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 129 0 0 0 KP_Enter\n"
                        "RenderEvent 168 129 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 128 0 0 0 KP_Enter\n"
                        "RenderEvent 168 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 128 0 0 0 KP_Enter\n"
                        "RenderEvent 169 128 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 127 0 0 0 KP_Enter\n"
                        "RenderEvent 169 127 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 126 0 0 0 KP_Enter\n"
                        "RenderEvent 169 126 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 125 0 0 0 KP_Enter\n"
                        "RenderEvent 170 125 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 125 0 0 0 KP_Enter\n"
                        "RenderEvent 171 125 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 124 0 0 0 KP_Enter\n"
                        "RenderEvent 171 124 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 123 0 0 0 KP_Enter\n"
                        "RenderEvent 172 123 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 123 0 0 0 KP_Enter\n"
                        "RenderEvent 173 123 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 122 0 0 0 KP_Enter\n"
                        "RenderEvent 173 122 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 121 0 0 0 KP_Enter\n"
                        "RenderEvent 173 121 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 121 0 0 0 KP_Enter\n"
                        "RenderEvent 174 121 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 120 0 0 0 KP_Enter\n"
                        "RenderEvent 174 120 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 175 120 0 0 0 KP_Enter\n"
                        "RenderEvent 175 120 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 120 0 0 0 KP_Enter\n"
                        "RenderEvent 176 120 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 119 0 0 0 KP_Enter\n"
                        "RenderEvent 176 119 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 178 119 0 0 0 KP_Enter\n"
                        "RenderEvent 178 119 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 178 118 0 0 0 KP_Enter\n"
                        "RenderEvent 178 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 178 118 0 0 0 KP_Enter\n"
                        "RenderEvent 178 118 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 178 117 0 0 0 KP_Enter\n"
                        "RenderEvent 178 117 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 178 117 0 0 0 KP_Enter\n"
                        "RenderEvent 178 117 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 177 117 0 0 0 KP_Enter\n"
                        "RenderEvent 177 117 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 117 0 0 0 KP_Enter\n"
                        "RenderEvent 176 117 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 176 117 0 0 0 KP_Enter\n"
                        "RenderEvent 176 117 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 175 117 0 0 0 KP_Enter\n"
                        "RenderEvent 175 117 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 175 117 0 0 0 KP_Enter\n"
                        "RenderEvent 175 117 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 175 116 0 0 0 KP_Enter\n"
                        "RenderEvent 175 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 174 116 0 0 0 KP_Enter\n"
                        "RenderEvent 174 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 116 0 0 0 KP_Enter\n"
                        "RenderEvent 173 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 173 116 0 0 0 KP_Enter\n"
                        "RenderEvent 173 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 172 116 0 0 0 KP_Enter\n"
                        "RenderEvent 172 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 116 0 0 0 KP_Enter\n"
                        "RenderEvent 171 116 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 115 0 0 0 KP_Enter\n"
                        "RenderEvent 171 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 171 115 0 0 0 KP_Enter\n"
                        "RenderEvent 171 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 170 115 0 0 0 KP_Enter\n"
                        "RenderEvent 170 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 115 0 0 0 KP_Enter\n"
                        "RenderEvent 169 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 115 0 0 0 KP_Enter\n"
                        "RenderEvent 169 115 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 114 0 0 0 KP_Enter\n"
                        "RenderEvent 169 114 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 114 0 0 0 KP_Enter\n"
                        "RenderEvent 169 114 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 169 113 0 0 0 KP_Enter\n"
                        "RenderEvent 169 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 113 0 0 0 KP_Enter\n"
                        "RenderEvent 168 113 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 112 0 0 0 KP_Enter\n"
                        "RenderEvent 168 112 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 111 0 0 0 KP_Enter\n"
                        "RenderEvent 168 111 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 111 0 0 0 KP_Enter\n"
                        "RenderEvent 168 111 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 110 0 0 0 KP_Enter\n"
                        "RenderEvent 168 110 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 109 0 0 0 KP_Enter\n"
                        "RenderEvent 168 109 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 167 109 0 0 0 KP_Enter\n"
                        "RenderEvent 167 109 0 0 0 KP_Enter\n"
                        "RightButtonReleaseEvent 167 109 0 0 0 KP_Enter\n"
                        "RenderEvent 167 109 0 0 0 KP_Enter\n"
                        "MouseMoveEvent 168 109 0 0 0 KP_Enter\n"
                        "RenderEvent 168 109 0 0 0 KP_Enter\n";

int TestLightWidget(int, char*[])
{
  // Create a renderer, render window and interactor
  vtkNew<vtkRenderer> ren;
  ren->SetBackground(0.1, 0.2, 0.4);
  vtkNew<vtkRenderWindow> renWin;
  renWin->SetSize(300, 300);
  renWin->AddRenderer(ren);

  vtkNew<vtkRenderWindowInteractor> iren;
  iren->SetRenderWindow(renWin);

  vtkNew<vtkLightRepresentation> lightRep;
  lightRep->SetPositional(true);

  vtkNew<vtkLightWidget> lightWidget;
  lightWidget->SetInteractor(iren);
  lightWidget->SetRepresentation(lightRep);
  lightWidget->On();

  vtkNew<vtkLightRepresentation> lightRep2;
  double color[3] = { 1.0, 1.0, 0.0 };
  lightRep2->SetLightColor(color);

  vtkNew<vtkLightWidget> lightWidget2;
  lightWidget2->SetInteractor(iren);
  lightWidget2->SetRepresentation(lightRep2);
  lightWidget2->On();

  renWin->SetMultiSamples(0);
  renWin->Render();
  ren->ResetCamera();
  renWin->Render();

  vtkNew<vtkInteractorStyleTrackballCamera> style;
  iren->Initialize();
  iren->SetInteractorStyle(style);

  vtkNew<vtkInteractorEventRecorder> recorder;
  recorder->SetInteractor(iren);
#if 0
  recorder->SetFileName("./record.log");
  recorder->Record();
  recorder->On();
#else
  recorder->ReadFromInputStringOn();
  recorder->SetInputString(eventLog);
  recorder->Play();
#endif

  iren->Start();

  recorder->Stop();

  return EXIT_SUCCESS;
}
