// python wrapper for vtkConditionVariable
//
#define VTK_WRAPPING_CXX
#define VTK_STREAMS_FWD_ONLY
#include "vtkPythonArgs.h"
#include "vtkPythonOverload.h"
#include <cstddef>
#include <sstream>
#include "vtkVariant.h"
#include "vtkIndent.h"
#include "vtkConditionVariable.h"

#if defined(PYTHON_PACKAGE)
#define PYTHON_PACKAGE_SCOPE PYTHON_PACKAGE "."
#else
#define PYTHON_PACKAGE_SCOPE
#endif

extern "C" { VTK_ABI_EXPORT void PyVTKAddFile_vtkConditionVariable(PyObject *dict); }

static PyObject *
PyvtkSimpleConditionVariable_Signal(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "Signal");
  void *vp = ap.GetSelfSpecialPointer(self, args);
  vtkSimpleConditionVariable *op = static_cast<vtkSimpleConditionVariable *>(vp);

  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(0))
  {
    op->Signal();

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildNone();
    }
  }

  return result;
}


static PyObject *
PyvtkSimpleConditionVariable_Broadcast(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "Broadcast");
  void *vp = ap.GetSelfSpecialPointer(self, args);
  vtkSimpleConditionVariable *op = static_cast<vtkSimpleConditionVariable *>(vp);

  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(0))
  {
    op->Broadcast();

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildNone();
    }
  }

  return result;
}


static PyObject *
PyvtkSimpleConditionVariable_Wait(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "Wait");
  void *vp = ap.GetSelfSpecialPointer(self, args);
  vtkSimpleConditionVariable *op = static_cast<vtkSimpleConditionVariable *>(vp);

  vtkSimpleMutexLock *temp0 = nullptr;
  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(1) &&
      ap.GetSpecialObject(temp0, "vtkSimpleMutexLock"))
  {
    int tempr = op->Wait(*temp0);

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildValue(tempr);
    }
  }

  return result;
}

static PyMethodDef PyvtkSimpleConditionVariable_Methods[] = {
  {"Signal", PyvtkSimpleConditionVariable_Signal, METH_VARARGS,
   "Signal(self) -> None\nC++: void Signal()\n\nWake one thread waiting for the condition to change.\n"},
  {"Broadcast", PyvtkSimpleConditionVariable_Broadcast, METH_VARARGS,
   "Broadcast(self) -> None\nC++: void Broadcast()\n\nWake all threads waiting for the condition to change.\n"},
  {"Wait", PyvtkSimpleConditionVariable_Wait, METH_VARARGS,
   "Wait(self, mutex:vtkSimpleMutexLock) -> int\nC++: int Wait(vtkSimpleMutexLock &mutex)\n\nWait for the condition to change. Upon entry, the mutex must be\nlocked and the lock held by the calling thread. Upon exit, the\nmutex will be locked and held by the calling thread. Between\nentry and exit, the mutex will be unlocked and may be held by\nother threads.\n\n* @param mutex The mutex that should be locked on entry and will\n  be locked on exit (but not in between)\n* @retval Normally, this function returns 0. Should a thread be\n  interrupted by a signal, a non-zero value may be returned.\n"},
  {nullptr, nullptr, 0, nullptr}
};


static PyObject *
PyvtkSimpleConditionVariable_vtkSimpleConditionVariable(PyObject * /*unused*/, PyObject *args)
{
  PyErr_WarnEx(PyExc_DeprecationWarning,
    "Call to deprecated class vtkSimpleConditionVariable."
    " (" "Use std::condition_variable_any instead." ")"
    " -- Deprecated since version " "9.1.0" ".", 1);

  vtkPythonArgs ap(args, "vtkSimpleConditionVariable");

  PyObject *result = nullptr;

  if (ap.CheckArgCount(0))
  {
    vtkSimpleConditionVariable *op = new vtkSimpleConditionVariable();

    result = PyVTKSpecialObject_New("vtkSimpleConditionVariable", op);
  }

  return result;
}

static PyMethodDef PyvtkSimpleConditionVariable_vtkSimpleConditionVariable_Methods[] = {
  {nullptr, nullptr, 0, nullptr}
};


static const char *PyvtkSimpleConditionVariable_Doc =

  "vtkSimpleConditionVariable() -> vtkSimpleConditionVariable\nC++: vtkSimpleConditionVariable()\n""\n"
  "vtkSimpleConditionVariable - no description provided.\n\n"
;

static PyObject *
PyvtkSimpleConditionVariable_New(PyTypeObject *, PyObject *args, PyObject *kwds)
{
  if (kwds && PyDict_Size(kwds))
  {
    PyErr_SetString(PyExc_TypeError,
                    "this function takes no keyword arguments");
    return nullptr;
  }

  return PyvtkSimpleConditionVariable_vtkSimpleConditionVariable(nullptr, args);
}

static void PyvtkSimpleConditionVariable_Delete(PyObject *self)
{
  PyVTKSpecialObject *obj = (PyVTKSpecialObject *)self;
  delete static_cast<vtkSimpleConditionVariable *>(obj->vtk_ptr);
  PyObject_Del(self);
}

static Py_hash_t PyvtkSimpleConditionVariable_Hash(PyObject *self)
{
  return PyObject_HashNotImplemented(self);
}

#ifdef VTK_PYTHON_NEEDS_DEPRECATION_WARNING_SUPPRESSION
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
#endif

static PyTypeObject PyvtkSimpleConditionVariable_Type = {
  PyVarObject_HEAD_INIT(&PyType_Type, 0)
  PYTHON_PACKAGE_SCOPE "vtkCommonCore.vtkSimpleConditionVariable", // tp_name
  sizeof(PyVTKSpecialObject), // tp_basicsize
  0, // tp_itemsize
  PyvtkSimpleConditionVariable_Delete, // tp_dealloc
#if PY_VERSION_HEX >= 0x03080000
  0, // tp_vectorcall_offset
#else
  nullptr, // tp_print
#endif
  nullptr, // tp_getattr
  nullptr, // tp_setattr
  nullptr, // tp_compare
  PyVTKSpecialObject_Repr, // tp_repr
  nullptr, // tp_as_number
  nullptr, // tp_as_sequence
  nullptr, // tp_as_mapping
  PyvtkSimpleConditionVariable_Hash, // tp_hash
  nullptr, // tp_call
  nullptr, // tp_str
  PyObject_GenericGetAttr, // tp_getattro
  nullptr, // tp_setattro
  nullptr, // tp_as_buffer
  Py_TPFLAGS_DEFAULT, // tp_flags
  PyvtkSimpleConditionVariable_Doc, // tp_doc
  nullptr, // tp_traverse
  nullptr, // tp_clear
  nullptr, // tp_richcompare
  0, // tp_weaklistoffset
  nullptr, // tp_iter
  nullptr, // tp_iternext
  nullptr, // tp_methods
  nullptr, // tp_members
  nullptr, // tp_getset
  nullptr, // tp_base
  nullptr, // tp_dict
  nullptr, // tp_descr_get
  nullptr, // tp_descr_set
  0, // tp_dictoffset
  nullptr, // tp_init
  nullptr, // tp_alloc
  PyvtkSimpleConditionVariable_New, // tp_new
  PyObject_Del, // tp_free
  nullptr, // tp_is_gc
  nullptr, // tp_bases
  nullptr, // tp_mro
  nullptr, // tp_cache
  nullptr, // tp_subclasses
  nullptr, // tp_weaklist
  VTK_WRAP_PYTHON_SUPPRESS_UNINITIALIZED
};

#ifndef DECLARED_PyvtkSimpleConditionVariable_TypeNew
extern "C" { PyObject *PyvtkSimpleConditionVariable_TypeNew(); }
#define DECLARED_PyvtkSimpleConditionVariable_TypeNew
#endif

PyObject *PyvtkSimpleConditionVariable_TypeNew()
{
  PyTypeObject *pytype = PyVTKSpecialType_Add(
    &PyvtkSimpleConditionVariable_Type,
    PyvtkSimpleConditionVariable_Methods,
    PyvtkSimpleConditionVariable_vtkSimpleConditionVariable_Methods,
    nullptr);

  if ((pytype->tp_flags & Py_TPFLAGS_READY) != 0)
  {
    return (PyObject *)pytype;
  }

  PyType_Ready(pytype);
  return (PyObject *)pytype;
}

extern "C" { PyObject *PyvtkConditionVariable_ClassNew(); }

#ifndef DECLARED_PyvtkObject_ClassNew
extern "C" { PyObject *PyvtkObject_ClassNew(); }
#define DECLARED_PyvtkObject_ClassNew
#endif

static PyObject *
PyvtkConditionVariable_IsTypeOf(PyObject * /*unused*/, PyObject *args)
{
  PyErr_WarnEx(PyExc_DeprecationWarning,
    "Call to deprecated class vtkConditionVariable."
    " (" "Use std::condition_variable_any instead." ")"
    " -- Deprecated since version " "9.1.0" ".", 1);

  vtkPythonArgs ap(args, "IsTypeOf");

  const char *temp0 = nullptr;
  PyObject *result = nullptr;

  if (ap.CheckArgCount(1) &&
      ap.GetValue(temp0))
  {
    int tempr = vtkConditionVariable::IsTypeOf(temp0);

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildValue(tempr);
    }
  }

  return result;
}


static PyObject *
PyvtkConditionVariable_IsA(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "IsA");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkConditionVariable *op = static_cast<vtkConditionVariable *>(vp);

  const char *temp0 = nullptr;
  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(1) &&
      ap.GetValue(temp0))
  {
    int tempr = (ap.IsBound() ?
      op->IsA(temp0) :
      op->vtkConditionVariable::IsA(temp0));

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildValue(tempr);
    }
  }

  return result;
}


static PyObject *
PyvtkConditionVariable_SafeDownCast(PyObject * /*unused*/, PyObject *args)
{
  PyErr_WarnEx(PyExc_DeprecationWarning,
    "Call to deprecated class vtkConditionVariable."
    " (" "Use std::condition_variable_any instead." ")"
    " -- Deprecated since version " "9.1.0" ".", 1);

  vtkPythonArgs ap(args, "SafeDownCast");

  vtkObjectBase *temp0 = nullptr;
  PyObject *result = nullptr;

  if (ap.CheckArgCount(1) &&
      ap.GetVTKObject(temp0, "vtkObjectBase"))
  {
    vtkConditionVariable *tempr = vtkConditionVariable::SafeDownCast(temp0);

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildVTKObject(tempr);
    }
  }

  return result;
}


static PyObject *
PyvtkConditionVariable_NewInstance(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "NewInstance");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkConditionVariable *op = static_cast<vtkConditionVariable *>(vp);

  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(0))
  {
    vtkConditionVariable *tempr = (ap.IsBound() ?
      op->NewInstance() :
      op->vtkConditionVariable::NewInstance());

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildVTKObject(tempr);
      if (result && PyVTKObject_Check(result))
      {
        PyVTKObject_GetObject(result)->UnRegister(nullptr);
        PyVTKObject_SetFlag(result, VTK_PYTHON_IGNORE_UNREGISTER, 1);
      }
    }
    else if (tempr != nullptr)
    {
      ap.DeleteVTKObject(tempr);
    }
  }

  return result;
}


static PyObject *
PyvtkConditionVariable_GetNumberOfGenerationsFromBaseType(PyObject * /*unused*/, PyObject *args)
{
  PyErr_WarnEx(PyExc_DeprecationWarning,
    "Call to deprecated class vtkConditionVariable."
    " (" "Use std::condition_variable_any instead." ")"
    " -- Deprecated since version " "9.1.0" ".", 1);

  vtkPythonArgs ap(args, "GetNumberOfGenerationsFromBaseType");

  const char *temp0 = nullptr;
  PyObject *result = nullptr;

  if (ap.CheckArgCount(1) &&
      ap.GetValue(temp0))
  {
    long long tempr = vtkConditionVariable::GetNumberOfGenerationsFromBaseType(temp0);

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildValue(tempr);
    }
  }

  return result;
}


static PyObject *
PyvtkConditionVariable_GetNumberOfGenerationsFromBase(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "GetNumberOfGenerationsFromBase");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkConditionVariable *op = static_cast<vtkConditionVariable *>(vp);

  const char *temp0 = nullptr;
  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(1) &&
      ap.GetValue(temp0))
  {
    long long tempr = (ap.IsBound() ?
      op->GetNumberOfGenerationsFromBase(temp0) :
      op->vtkConditionVariable::GetNumberOfGenerationsFromBase(temp0));

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildValue(tempr);
    }
  }

  return result;
}


static PyObject *
PyvtkConditionVariable_Signal(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "Signal");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkConditionVariable *op = static_cast<vtkConditionVariable *>(vp);

  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(0))
  {
    if (ap.IsBound())
    {
      op->Signal();
    }
    else
    {
      op->vtkConditionVariable::Signal();
    }

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildNone();
    }
  }

  return result;
}


static PyObject *
PyvtkConditionVariable_Broadcast(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "Broadcast");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkConditionVariable *op = static_cast<vtkConditionVariable *>(vp);

  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(0))
  {
    if (ap.IsBound())
    {
      op->Broadcast();
    }
    else
    {
      op->vtkConditionVariable::Broadcast();
    }

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildNone();
    }
  }

  return result;
}


static PyObject *
PyvtkConditionVariable_Wait(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "Wait");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkConditionVariable *op = static_cast<vtkConditionVariable *>(vp);

  vtkMutexLock *temp0 = nullptr;
  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(1) &&
      ap.GetVTKObject(temp0, "vtkMutexLock"))
  {
    int tempr = (ap.IsBound() ?
      op->Wait(temp0) :
      op->vtkConditionVariable::Wait(temp0));

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildValue(tempr);
    }
  }

  return result;
}

static PyMethodDef PyvtkConditionVariable_Methods[] = {
  {"IsTypeOf", PyvtkConditionVariable_IsTypeOf, METH_VARARGS,
   "IsTypeOf(type:str) -> int\nC++: static vtkTypeBool IsTypeOf(const char *type)\n\nReturn 1 if this class type is the same type of (or a subclass\nof) the named class. Returns 0 otherwise. This method works in\ncombination with vtkTypeMacro found in vtkSetGet.h.\n"},
  {"IsA", PyvtkConditionVariable_IsA, METH_VARARGS,
   "IsA(self, type:str) -> int\nC++: vtkTypeBool IsA(const char *type) override;\n\nReturn 1 if this class is the same type of (or a subclass of) the\nnamed class. Returns 0 otherwise. This method works in\ncombination with vtkTypeMacro found in vtkSetGet.h.\n"},
  {"SafeDownCast", PyvtkConditionVariable_SafeDownCast, METH_VARARGS,
   "SafeDownCast(o:vtkObjectBase) -> vtkConditionVariable\nC++: static vtkConditionVariable *SafeDownCast(vtkObjectBase *o)\n\n"},
  {"NewInstance", PyvtkConditionVariable_NewInstance, METH_VARARGS,
   "NewInstance(self) -> vtkConditionVariable\nC++: vtkConditionVariable *NewInstance()\n\n"},
  {"GetNumberOfGenerationsFromBaseType", PyvtkConditionVariable_GetNumberOfGenerationsFromBaseType, METH_VARARGS,
   "GetNumberOfGenerationsFromBaseType(type:str) -> int\nC++: static vtkIdType GetNumberOfGenerationsFromBaseType(\n    const char *type)\n\nGiven a the name of a base class of this class type, return the\ndistance of inheritance between this class type and the named\nclass (how many generations of inheritance are there between this\nclass and the named class). If the named class is not in this\nclass's inheritance tree, return a negative value. Valid\nresponses will always be nonnegative. This method works in\ncombination with vtkTypeMacro found in vtkSetGet.h.\n"},
  {"GetNumberOfGenerationsFromBase", PyvtkConditionVariable_GetNumberOfGenerationsFromBase, METH_VARARGS,
   "GetNumberOfGenerationsFromBase(self, type:str) -> int\nC++: vtkIdType GetNumberOfGenerationsFromBase(const char *type)\n    override;\n\nGiven the name of a base class of this class type, return the\ndistance of inheritance between this class type and the named\nclass (how many generations of inheritance are there between this\nclass and the named class). If the named class is not in this\nclass's inheritance tree, return a negative value. Valid\nresponses will always be nonnegative. This method works in\ncombination with vtkTypeMacro found in vtkSetGet.h.\n"},
  {"Signal", PyvtkConditionVariable_Signal, METH_VARARGS,
   "Signal(self) -> None\nC++: void Signal()\n\nWake one thread waiting for the condition to change.\n"},
  {"Broadcast", PyvtkConditionVariable_Broadcast, METH_VARARGS,
   "Broadcast(self) -> None\nC++: void Broadcast()\n\nWake all threads waiting for the condition to change.\n"},
  {"Wait", PyvtkConditionVariable_Wait, METH_VARARGS,
   "Wait(self, mutex:vtkMutexLock) -> int\nC++: int Wait(vtkMutexLock *mutex)\n\nWait for the condition to change. Upon entry, the mutex must be\nlocked and the lock held by the calling thread. Upon exit, the\nmutex will be locked and held by the calling thread. Between\nentry and exit, the mutex will be unlocked and may be held by\nother threads.\n\n* @param mutex The mutex that should be locked on entry and will\n  be locked on exit (but not in between)\n* @retval Normally, this function returns 0. Should a thread be\n  interrupted by a signal, a non-zero value may be returned.\n"},
  {nullptr, nullptr, 0, nullptr}
};

static const char *PyvtkConditionVariable_Doc =
  "vtkConditionVariable - mutual exclusion locking class\n\n"
  "Superclass: vtkObject\n\n"
  "vtkConditionVariable allows the locking of variables which are\n"
  "accessed through different threads.  This header file also defines\n"
  "vtkSimpleConditionVariable which is not a subclass of vtkObject.\n\n"
  "The win32 implementation is based on notes provided by Douglas C.\n"
  "Schmidt and Irfan Pyarali, Department of Computer Science, Washington\n"
  "University, St. Louis, Missouri.\n"
  "http://www.cs.wustl.edu/~schmidt/win32-cv-1.html\n\n";

#ifdef VTK_PYTHON_NEEDS_DEPRECATION_WARNING_SUPPRESSION
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
#endif

static PyTypeObject PyvtkConditionVariable_Type = {
  PyVarObject_HEAD_INIT(&PyType_Type, 0)
  PYTHON_PACKAGE_SCOPE "vtkCommonCore.vtkConditionVariable", // tp_name
  sizeof(PyVTKObject), // tp_basicsize
  0, // tp_itemsize
  PyVTKObject_Delete, // tp_dealloc
#if PY_VERSION_HEX >= 0x03080000
  0, // tp_vectorcall_offset
#else
  nullptr, // tp_print
#endif
  nullptr, // tp_getattr
  nullptr, // tp_setattr
  nullptr, // tp_compare
  PyVTKObject_Repr, // tp_repr
  nullptr, // tp_as_number
  nullptr, // tp_as_sequence
  nullptr, // tp_as_mapping
  nullptr, // tp_hash
  nullptr, // tp_call
  PyVTKObject_String, // tp_str
  PyObject_GenericGetAttr, // tp_getattro
  PyObject_GenericSetAttr, // tp_setattro
  &PyVTKObject_AsBuffer, // tp_as_buffer
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_BASETYPE, // tp_flags
  PyvtkConditionVariable_Doc, // tp_doc
  PyVTKObject_Traverse, // tp_traverse
  nullptr, // tp_clear
  nullptr, // tp_richcompare
  offsetof(PyVTKObject, vtk_weakreflist), // tp_weaklistoffset
  nullptr, // tp_iter
  nullptr, // tp_iternext
  nullptr, // tp_methods
  nullptr, // tp_members
  PyVTKObject_GetSet, // tp_getset
  nullptr, // tp_base
  nullptr, // tp_dict
  nullptr, // tp_descr_get
  nullptr, // tp_descr_set
  offsetof(PyVTKObject, vtk_dict), // tp_dictoffset
  nullptr, // tp_init
  nullptr, // tp_alloc
  PyVTKObject_New, // tp_new
  PyObject_GC_Del, // tp_free
  nullptr, // tp_is_gc
  nullptr, // tp_bases
  nullptr, // tp_mro
  nullptr, // tp_cache
  nullptr, // tp_subclasses
  nullptr, // tp_weaklist
  VTK_WRAP_PYTHON_SUPPRESS_UNINITIALIZED
};

static vtkObjectBase *PyvtkConditionVariable_StaticNew()
{
  PyErr_WarnEx(PyExc_DeprecationWarning,
    "Call to deprecated class vtkConditionVariable."
    " (" "Use std::condition_variable_any instead." ")"
    " -- Deprecated since version " "9.1.0" ".", 1);

  return vtkConditionVariable::New();
}

PyObject *PyvtkConditionVariable_ClassNew()
{
  PyTypeObject *pytype = PyVTKClass_Add(
    &PyvtkConditionVariable_Type, PyvtkConditionVariable_Methods,
    "vtkConditionVariable",
 &PyvtkConditionVariable_StaticNew);

  if ((pytype->tp_flags & Py_TPFLAGS_READY) != 0)
  {
    return (PyObject *)pytype;
  }

#ifndef VTK_PY3K
  pytype->tp_flags |= Py_TPFLAGS_HAVE_NEWBUFFER;
#endif

  pytype->tp_base = (PyTypeObject *)PyvtkObject_ClassNew();

  PyType_Ready(pytype);
  return (PyObject *)pytype;
}

void PyVTKAddFile_vtkConditionVariable(
  PyObject *dict)
{
  PyObject *o;
  o = PyvtkSimpleConditionVariable_TypeNew();

  if (o && PyDict_SetItemString(dict, "vtkSimpleConditionVariable", o) != 0)
  {
    Py_DECREF(o);
  }

  o = PyvtkConditionVariable_ClassNew();

  if (o && PyDict_SetItemString(dict, "vtkConditionVariable", o) != 0)
  {
    Py_DECREF(o);
  }

}

